/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view.AccuracyTests;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.util.Set;

import com.topcoder.rules.dependencymap.view.ShapeFactory;


/**
 *  Helper class to manually build a diagram.
 *
 * @author cucu
 */
public class GraphicsHelper {

    private ShapeFactory sf = null;
    private MockGraphics graphics = null;
    private int shapeWidth = 0;
    private int shapeHeight = 0;
    private int colPadding = 0;
    private int rowPadding = 0;
    private Insets margin = null;
    private int arrowHeight = 0;
    private Color textColor = Color.BLACK;
    private Color shapeFillColor = Color.WHITE;
    private Color shapeLineColor = Color.BLACK;
    private Color arrowLineColor = Color.BLACK;
    private Color arrowFillColor = Color.WHITE;
    private boolean enhancement2 = false;
    private int maxHeight = 0;
    private int maxWidth = 0;


    /**
     * get the graphics used for drawing
     *
     * @return the graphics used for drawing
     */
    public MockGraphics getGraphics() {
        return graphics;
    }

    /**
     *  Build a new Helper.
     *
     */
    public GraphicsHelper(ShapeFactory shapes, MockGraphics graphics, int colPadding, int rowPadding, Insets margin)
    {
        sf = shapes;
        this.graphics = graphics;
        this.colPadding = colPadding;
        this.rowPadding = rowPadding;
        this.margin = margin;
        arrowHeight = shapes.getShape(ShapeFactory.RELATIONSHIP_ARROW).getBounds().height;
    }

    /**
     * Convert from grid coordinates to pixel coordinates
     */
    private Point convert(double x, double y) {
        return new Point((int) Math.round((x - 1) * (shapeWidth + colPadding)),
                         (int) Math.round((y - 1) * (shapeHeight + arrowHeight + rowPadding)));
    }

    /**
     * Draw a seen/unseen shape with the specified text in the grid coordinates
     *
     * @param seen true for seen shape
     * @param x x in grid coordinates, 1-based
     * @param y y in grid coordinates, 1-based
     * @param text text to write centered
     */
    public void drawShape(boolean seen, double x, int y, String text) {
        Point p = convert(x,y);
        drawShape(seen, p.x, p.y, shapeWidth, shapeHeight, text);
    }


    /**
     * Draw an arrow that points from the x0,y0 grid coordinates to x1,y1 grid coordinates.
     *
     * @param x0 start x in grid coordinates, 1-based
     * @param y0 start y in grid coordinates, 1-based
     * @param x1 end x in grid coordinates, 1-based
     * @param y1 end y in grid coordinates, 1-based
     */
    public void drawArrow(double x0, int y0, double x1, int y1) {
        Point p0 =  convert(x0, y0);
        Point p1 =  convert(x1, y1);
        p0.x += shapeWidth / 2;
        p0.y += shapeHeight / 2;
        p1.x += shapeWidth / 2;
        p1.y += shapeHeight / 2;
        drawArrow(p0, p1);
    }

    /**
     * Draw a shape in the specified pixel coordinates.
     *
     * @param seen true for seen shape
     * @param x left coordinate in pixels
     * @param y upper coordinate in pixels<
     * @param width width of the shape
     * @param height height of the shape
     * @param text text to write centered in the shape
     */
    private void drawShape(boolean seen, int x, int y, int width, int height, String text) {
        GeneralPath path = (GeneralPath) sf.getShape(seen? ShapeFactory.SEEN_ELEMENT : ShapeFactory.UNSEEN_ELEMENT);
        Rectangle bounds = path.getBounds();

        if ((x + width) > maxWidth) {
            maxWidth = ((int) x) + width;
        }
        if ((y + height) > maxHeight) {
            maxHeight = y + height;
        }

        path.transform(AffineTransform.getScaleInstance(width / bounds.getWidth(), height / bounds.getHeight()));
        path.transform(AffineTransform.getTranslateInstance(x, y));

        FontMetrics fm = graphics.getFontMetrics(graphics.getFont());
        Point textPos = new Point(((int) x) + (width + margin.left - margin.right - fm.stringWidth(text)) / 2,
                                  y + (height + margin.top - margin.bottom + fm.getAscent() - fm.getDescent()) / 2);

        if (enhancement2) {
            graphics.setColor(shapeFillColor);
            graphics.fill(path);
            graphics.setColor(shapeLineColor);
            graphics.draw(path);
            graphics.setColor(textColor);
            //graphics.setFont(font);
            graphics.drawString(text, textPos.x, textPos.y);
        } else {
            graphics.draw(path);
            graphics.drawString(text, textPos.x, textPos.y);
        }

    }

    /**
     * Draw an arrow that points from the p0 grid coordinates to p1 grid coordinates.
     *
     * @param p1 start point
     * @param p2 end point
     */
    private void drawArrow(Point p1, Point p2) {
        Shape shape = sf.getShape(ShapeFactory.RELATIONSHIP_ARROW);
        Rectangle bounds = shape.getBounds();
        double alpha = Math.atan2(p2.y - p1.y, p2.x - p1.x) - Math.PI / 2;
        float dx = (p1.x + p2.x) / 2;
        float dy = (p1.y + p2.y) / 2;

        GeneralPath path = (GeneralPath) shape;

        path.transform(AffineTransform.getRotateInstance(alpha, (bounds.width / 2 + bounds.x), (bounds.height / 2 + bounds.y)));
        path.transform(AffineTransform.getTranslateInstance(dx - (bounds.width / 2 + bounds.x), dy - (bounds.height / 2 + bounds.y)));

        if (enhancement2) {
            graphics.setColor(arrowFillColor);
            graphics.fill(path);
            graphics.setColor(arrowLineColor);
            graphics.draw(path);
        } else {
            graphics.draw(path);
        }
    }

    /**
     * Set the shape size so that it fits the text.
     *
     * @param text text to fit
     */
    public void setShapeSizeTo(String text) {
        FontMetrics fm = graphics.getFontMetrics(graphics.getFont());
        shapeWidth = fm.stringWidth(text) + margin.left + margin.right  + 2;
        shapeHeight = fm.getHeight() + margin.top + margin.bottom + 2;
    }

    /**
     * Get the log created by the mock graphics
     * @return
     */
    public Set getLog() {
        return graphics.getLog();
    }

    public int getWidth() {
        return maxWidth;
    }

    public int getHeight() {
        return maxHeight;
    }

    public void setColors(Color text, Color shapeFill, Color shapeLine, Color arrowLine, Color arrowFill) {
        textColor = text;
        shapeFillColor = shapeFill;
        shapeLineColor = shapeLine;
        arrowFillColor = arrowFill;
        arrowLineColor = arrowLine;

        enhancement2 = true;
    }

    public void setFont(Font font) {
        graphics.setFont(font);
        enhancement2 = true;
    }

}
